/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.databinding.jaxb;

import java.util.HashMap;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import org.ow2.petals.databinding.jaxb.tools.DataBindingConstants;

/**
 * Registry to save the instances used to Marshall/UnMarshall operations messages.
 * @author Mohammed EL JAI - EBM WebSourcing
 */
public class Registry {
    
    private Map<String, MarshallingObjects> marshallingObjects;
    
    public Registry() {
        this.setMarshallingObjects(new HashMap<String, MarshallingObjects>());
    }

    public void setMarshallingObjects(Map<String, MarshallingObjects> marshallingObjects) {
        this.marshallingObjects = marshallingObjects;
    }

    public Map<String, MarshallingObjects> getMarshallingObjects() {
        return marshallingObjects;
    }
    
    public void addEntry(String nameSpace, String packageName, ClassLoader jaxbClassLoader) throws Exception {
        
        MarshallingObjects marshallingObjects = new MarshallingObjects();
        
        JAXBContext jaxbContext = JAXBContext.newInstance(packageName, jaxbClassLoader);
        
        Class<?> jaxbFactoryClass = 
            jaxbClassLoader.loadClass(packageName + "." + DataBindingConstants.JAXB_FACTORY_CLASS_NAME); 
        marshallingObjects.setJaxbFactoryClass(jaxbFactoryClass);
        
        Marshaller marshaller = jaxbContext.createMarshaller();
        marshallingObjects.setMarshaller(marshaller);
        
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        marshallingObjects.setUnmarshaller(unmarshaller);
        
        this.getMarshallingObjects().put(nameSpace, marshallingObjects);
                
    }
    
    public Marshaller getMarshaller(String nameSpace) {
        return marshallingObjects.get(nameSpace).getMarshaller();
    }
    
    public Unmarshaller getUnmarshaller(String nameSpace) {
        return  marshallingObjects.get(nameSpace).getUnmarshaller();
    }

    public Object getJaxbFactoryInstance(String nameSpace) {
        return marshallingObjects.get(nameSpace).getJaxbFactoryInstnace();
    }

  



    private class MarshallingObjects {
        
        /** **/
        private Marshaller marshaller;
        
        /** **/
        private Unmarshaller unmarshaller;
        
        /** **/
        private Class<?> jaxbFactoryClass; 
        
        private Object jaxbFactoryInstnace;
        
        public Marshaller getMarshaller() {
            return marshaller;
        }

        public void setMarshaller(Marshaller marshaller) {
            this.marshaller = marshaller;
        }

        public Unmarshaller getUnmarshaller() {
            return unmarshaller;
        }

        public void setUnmarshaller(Unmarshaller unmarshaller) {
            this.unmarshaller = unmarshaller;
        }

        @SuppressWarnings("unused")
        public Class<?> getJaxbFactoryClass() {
            return jaxbFactoryClass;
        }

        public void setJaxbFactoryClass(Class<?> jaxbFactoryClass) {
            try {
                this.setJaxbFactoryInstnace(jaxbFactoryClass.newInstance());
            } catch (InstantiationException e) {
            } catch (IllegalAccessException e) {
            }
            this.jaxbFactoryClass = jaxbFactoryClass;
        }

        public void setJaxbFactoryInstnace(Object jaxbFactoryInstnace) {
            this.jaxbFactoryInstnace = jaxbFactoryInstnace;
        }

        public Object getJaxbFactoryInstnace() {
            return jaxbFactoryInstnace;
        }
        
    }
     

}
