/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.databinding.jaxb.tools;

import java.io.File;
import java.util.List;

import com.sun.tools.xjc.Driver;

/**
 * This class offers some utilities to generate and compile the JAXB classes.
 * @author Mohammed EL JAI - EBM WebSourcing 
 */
public class JaxbClassGenerator {
	
	/**
	 * Generate the JAXB classes for the WSDL given in parameter 
	 * @param wsdlUrl : The WSDL URL
	 * @param rootPath : The base directory of the generation
	 * @param genDirName : The name of the directory where the sources will be generated 
	 */
	public static void  generateJaxbClass(String wsdlUrl, String rootPath, String genDirName) {
		try {
			String generatedDir = rootPath + File.separator + genDirName;
			Util.createDirectory(generatedDir);
			Driver.run(createJxcOptions(wsdlUrl,generatedDir), new PEtALSXjcListener());	
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Compile the generated JAXB classes 
	 * @param baseDirPath : The base directory of the compilation
	 * @param srcDirName : The source directory name
	 * @param classPath : List of class path to compile the source file
	 */
	public static void compileJaxbClass(String baseDirPath, String srcDirName, List<String> classPath) {
		
    	try {
    		DataBindingCompiler.compile(baseDirPath, srcDirName, classPath);
		} catch (Exception e) {
			e.printStackTrace();
		}
    	
		
	}
	
	
	/**
	 * Create option for JAXB to generate databinding classes 
	 * @param wsdlUrl : The WSDL URL
	 * @param generatedDir : The name of the directory where the sources will be generated
	 * @return a list of options
	 */
	private static String[] createJxcOptions(String wsdlUrl, String generatedDir) {
		
		String[] options = new String[4];
		options[0] = "-wsdl";
		options[1] = wsdlUrl;
		options[2] = "-d";
		options[3] = generatedDir;
		
		return options;
	}
	
	

}
